local modName = "Totems"
local parent = HudMap
local L = LibStub("AceLocale-3.0"):GetLocale("HudMap")
local mod = HudMap:NewModule(modName, "AceEvent-3.0")

local db

local totemRanges = {
	-- Fire
	{
		[3599]  = 30,	-- Searing
		[8181]  = 30,	-- Frost Resist
		[8190]  = 8,		-- Magma
		[8227]  = 30,	-- Flametongue
		[30706] = 40,	-- Totem of Wrath
	},
	
	-- Earth
	{
		[2484]  = 10,	-- Earthbind
		[5730]  = 8,		-- Stoneclaw
		[8071]  = 30,	-- Stoneskin
		[8075]  = 8,		-- Strength of Earth
		[8143]  = 30,	-- Tremor
	},	
	
	-- Water
	{
		[5675]  = 30,	-- Mana Spring
		[8170]  = 30,	-- Cleansing
		[8184]  = 30,	-- Fire Resist
		[16190] = 30,	-- Mana Tide
		[5394]  = 30,	-- Healing Stream
	},
	
	-- Wind
	{
		[3738] = 30,	-- Wrath of Air
		[8177]  = 20,	-- Grounding
		[8512]  = 30,	-- Windfury
		[10595] = 30,	-- Nature Resist
	},
	
	--[[ Heart
	{
		[70575] = 9001
	}
	]]--
}

local options = {
	type = "group",
	name = L["Totems"],
	disabled = function()
		local _, c = UnitClass("player")
		return c ~= "SHAMAN"
	end,
	args = {
		enable = {
			type = "toggle",
			name = L["Enable Totem Ranges"],
			order = 100,
			get = function()
				return db.enable
			end,
			set = function(info, v)
				db.enable = v
				mod:UpdateGatherMode()
			end,
			disabled = isShaman,
		}
	}
}

totemCircles = {}
local iconToId = {}
local groups = {
	L["Fire Totems"],
	L["Earth Totems"],
	L["Water Totems"],
	L["Air Totems"]
}

local defaults = {
	profile = {
		enable = true
	}
}

local function getSlotForTotem(totem)
	for index, set in ipairs(totemRanges) do
		for id, range in pairs(set) do
			if id == totem then
				return index
			end
		end
	end
end

for index, v in ipairs(totemRanges) do
	local optGroup = {
		name = groups[index],
		type = 'group',
		args = {}
	}
	for id, range in pairs(v) do
		local n = GetSpellInfo(id)
		local sid = tostring(id)
		defaults.profile[sid] = true
		optGroup.args[sid] = {
			name = n,
			type = "toggle",
			get = function()
				return db[sid]
			end,
			set = function(info, v)
				db[sid] = v
				if not v then
					local x = totemCircles[tostring(getSlotForTotem(id))]
					if x then x:Free() end
				end
			end
		}		
	end
	options.args[groups[index]:gsub(" ", "")] = optGroup
end


local colors = {
	{1, 36/255, 0, 0.7},
	{200/255, 121/255, 43/255, 0.7},
	{0, 138/255, 1, 0.7},
	{150/255, 1, 0, 0.7}	
}

local effectiveTotemRanges = {}

local talents = {
	earths_grasp = {totem_ids = {[2484] = 1}, range_per_point = 10, tab_index = 2, talent_index = 2}
}
local offsets = {
	{-1, 1},
	{1, 1},
	{1, -1},
	{-1, -1},
}

function mod:OnInitialize()
	self.db = parent.db:RegisterNamespace(modName, defaults)
	db = self.db.profile
	parent:RegisterModuleOptions(modName, options, modName)
end

function mod:OnEnable()
	self:RegisterEvent("PLAYER_TOTEM_UPDATE")
	self:RegisterEvent("PLAYER_TALENT_UPDATE", "UpdateTotemInformation")
	self:UpdateTotemInformation()
end

function mod:UpdateTotemInformation()
	wipe(effectiveTotemRanges)
	wipe(iconToId)
	for set, totems in ipairs(totemRanges) do
		for id, range in pairs(totems) do 
			local name, _, icon = GetSpellInfo(id)
			for talent_name, talent_data in pairs(talents) do
				if talent_data.totem_ids[id] then
					local points = select(5, GetTalentInfo(talent_data.tab_index, talent_data.talent_index))
					range = range + (points * talent_data.range_per_point)
				end
			end
			iconToId[icon] = id
			effectiveTotemRanges[icon] = range
		end
	end
end

function mod:PLAYER_TOTEM_UPDATE(event, slot)
	local haveTotem, totemName, startTime, duration, icon = GetTotemInfo(slot)
	if totemCircles[tostring(slot)] then
		totemCircles[tostring(slot)] = totemCircles[tostring(slot)]:Free()
	end
	local range = effectiveTotemRanges[icon]
	local enabled = db.enable and db[tostring(iconToId[icon])]
	if haveTotem and range and enabled then
		local x, y = HudMap:GetUnitPosition("player")
		local r, g, b, a = unpack(colors[slot])
		local offset = offsets[slot]
		
		local bearing = -GetPlayerFacing()
		local angle = math.atan2(offset[1], offset[2])
		local hyp = 2.9 -- math.sqrt(8)
		local nx, ny = math.sin(angle + bearing), math.cos(angle + bearing)
		x = x + (nx * hyp)
		y = y + (ny * hyp)
		
		totemCircles[tostring(slot)] = HudMap:PlaceRangeMarker("radius", x, y, range*2, nil, r, g, b, a):Rotate(360 * (slot % 2 == 0 and 1 or -1), 40+(slot*3)):Appear()
	end
end